/*
 * $Id$
 */
package simulation;

import java.util.Vector;

import Jare.JareEngine;
import Jare.JareEnv;
import Jare.JareParseError;
import Jare.JareParser;

/**
 * @author Kai Sterker
 *
 */
public class Agent {

	private Environment environment;
	private int zeit = 0;
	private JareEngine wissensbasis = new JareEngine ();
	private int posY;
	private int posX;
	private int orientierung;

	/**
	 * @param x
	 * @param y
	 */
	public Agent (int x, int y) {
		wissensbasis.readKBfile("wumpus_agent.txt");
		// wissensbasis.setTrace (true);
		
		this.posX = x;
		this.posY = y;
		this.orientierung = 0;
	}
	
	/**
	 * @return
	 */
	public int getPosX () {
		return posX;
	}

	/**
	 * @return
	 */
	public int getPosY () {
		return posY;
	}

	/**
	 * Im Gegensatz zur Literatur erhaelt der Agent seine neue
	 * Wahrnehmung am Ende jeder Runde, damit der jeweilig
	 * aktuelle Wissensstand in der GUI angezeigt werden kann.
     * Daher sind nehmeWahr und update entkoppelt.
	 * @param w
	 */
	public void nehmeWahr (Wahrnehmung w) {
		System.out.println ("Percept: " + w.toAusdruck (zeit));			
		wissensbasis.addFactStr (w.toAusdruck (zeit));
	}
	
	/**
	 * Neuen Sinneseindruck wahrnehmen, d.h. der Agent lernt etwas
	 * ueber seine Umgebung dazu.
	 * @param w
	 */
	public Aktion update () {
		Aktion aktion = this.agiere ();
		System.out.println ("Aktion: " + aktion.toAusdruck (zeit) + "\n");
		wissensbasis.addFactStr (aktion.toAusdruck (zeit));
		zeit++;
		
		return aktion;
	}

	/**
	 * Eigentliches "Hirn" des Agenten. Sagt ihm aufgrund seines
	 * Wissens was er als naechstes tun soll.
	 * @return
	 */
	private Aktion agiere () {
		try {
			String qstr = new String ("(action ?a " + zeit + ")");
			Vector query = JareParser.parse (qstr);
			JareEnv solution = wissensbasis.query (query);
			if (solution == null) return zufallsAktion ();
			System.out.println ("Q-Result: " + solution.lookup("?a"));

			return new Aktion (solution);
		} catch (JareParseError e) {
			System.out.println ("Query failed!");
			e.printStackTrace ();
		}
		return null;
	}
	
	/**
	 * @return
	 */
	private Aktion zufallsAktion() {
		System.out.println ("Q-Result: nil");
		return new Aktion ("forward");
	}

	public void forward () {
		switch (orientierung) {
			case 0: posX--; break;
			case 90: posY--; break;
			case 180: posX++; break;
			case 270: posY++; break;
		}
		updateStatus ();
	}
	
	public void turn_left () {
		orientierung = (orientierung + 270) % 360;
		updateStatus ();
	}
	
	public void turn_right () {
		orientierung = (orientierung + 90) % 360;
		updateStatus ();
	}

	public void updateStatus () {
		// neue position bekanntmachen
		wissensbasis.addFactStr (
			"(at agent " + posX + " " + posY + " " + zeit +")");
		wissensbasis.addFactStr (
			"(orientation " + orientierung + " " + zeit +")");
	
		// wissensbasis.printKB();
	}
	
	public String toString () {
		switch (orientierung) {
			case 0: return "agent_n";
			case 90: return "agent_w";
			case 180: return "agent_s";
			case 270: return "agent_e";
		}
		return "error";
	}

	/**
	 * @return
	 */
	public int getOrientierung() {
		return orientierung;
	}

	/**
	 * @param x
	 * @param y
	 * @return
	 */
	public Feld wissen(int x, int y) {
		Feld f = new Feld (0);
		JareEnv solution;
		Vector query;
		
		try {
			String qstr = "(at ?objekt " + x + " " + y + " " + (zeit-1) + ")";
			query = JareParser.parse (qstr);
			solution = wissensbasis.query (query);
			
			while (solution != null) {
				f.add ((String) solution.lookup ("?objekt"));
				solution = wissensbasis.next();
			}
		} catch (JareParseError e) {
			System.out.println ("Query failed!");
			e.printStackTrace ();
		}
		
		if (x == posX && y == posY) f.add (toString());
		return f;
	}

	/**
	 * @param x
	 * @param y
	 * @return
	 */
	public boolean dangerAt(int x, int y) {
		try {
			Vector query = JareParser.parse("(ok " + x + " " + y + ")");
			return wissensbasis.query(query) == null;
		} catch (JareParseError e) {
			e.printStackTrace();
		}
		return true;
	}

	/**
	 * @param x
	 * @param y
	 * @return
	 */
	public boolean visited (int x, int y) {
		try {
			Vector query = JareParser.parse("(visited " + x + " " + y + ")");
			return wissensbasis.query(query) != null;
		} catch (JareParseError e) {
			e.printStackTrace();
		}

		return false;
	}
}
